/*
 * FlightPlanner.java
 */

package demos;

import acm.program.*;
import acm.util.*;
import java.io.*;
import java.util.*;

@SuppressWarnings("serial")
public class FlightPlanner extends ConsoleProgram {

	public void run() {
		println("Welcome to Flight Planner.");
		readFlightData("flights.txt");
		println("Cities in the database:");
		printCityList(cities);
		println("Plan a round-trip route --");
		String startCity = readLine("Enter the starting city: ");
		ArrayList route = new ArrayList();
		route.add(startCity);
		String currentCity = startCity;
		while (true) {
			String nextCity = getNextCity(currentCity);
			route.add(nextCity);
			if (nextCity.equals(startCity)) break;
			currentCity = nextCity;
		}
		printRoute(route);
	}
	
	private String getNextCity(String city) {
		ArrayList destinations = getDestinations(city);
		String nextCity = null;
		while (true) {
			println("From " + city + " there are direct flights to:");
			printCityList(destinations);
			String prompt = "Where do you want to go from " + city + "? ";
			nextCity = readLine(prompt);
			if (destinations.contains(nextCity)) break;
			println("There is no direct flight from " + city + " to " + nextCity + ".");
		}
		return nextCity;
	}
	
	private ArrayList getDestinations(String fromCity) {
		return (ArrayList) flights.get(fromCity);
	}
	
	private void printCityList(ArrayList cityList) {
		for (int i = 0; i < cityList.size(); i++) {
			String city = (String) cityList.get(i);
			println("  " + city);
		}
	}
	
	private void printRoute(ArrayList route) {
		println("The route that you have chosen is: ");
		for (int i = 0; i < route.size(); i++) {
			if (i > 0) print(" -> ");
			print((String) route.get(i));
		}
		println();
	}
	
	private void readFlightData(String filename) {
		flights = new HashMap();
		cities = new ArrayList();
		try {
			BufferedReader rd = new BufferedReader(new FileReader(filename));
			while (true) {
				String line = rd.readLine();
				if (line == null) break;
				if (line.length() != 0) readFlightEntry(line);
			}
			rd.close();
		} catch (IOException ex) {
			throw new ErrorException(ex);
		}
	}
	
	private void readFlightEntry(String line) {
		int arrow = line.indexOf("->");
		if (arrow == -1) {
			throw new ErrorException("Illegal flight entry " + line);
		}
		String fromCity = line.substring(0, arrow).trim();
		String toCity = line.substring(arrow + 2).trim();
		defineCity(fromCity);
		defineCity(toCity);
		getDestinations(fromCity).add(toCity);
	}
	
	private void defineCity(String cityName) {
		if (!cities.contains(cityName)) {
			cities.add(cityName);
			flights.put(cityName, new ArrayList());
		}
	}
	
	private HashMap flights;
	private ArrayList cities;
}
